# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""
fVersion = open('VERSION', 'r+').readline().rstrip()

CoreConfig = {
    'info': {
        'name': 'Opendoor scanner',
        'repository': 'git@github.com:stanislav-web/OpenDoor.git',
        'remote_version': 'https://raw.githubusercontent.com/stanislav-web/OpenDoor/master/VERSION',
        'license': 'License: GNU General Public License',
        'version': fVersion,
        'documentation': 'https://opendoor.readthedocs.org',
        'required_versions': {
            'minor': '3.7',
            'major': '3.11'
        },
    },
    'data': {
        'directories': 'data/directories.dat',
        'ignored': 'data/ignored.dat',
        'proxies': 'data/proxies.dat',
        'subdomains': 'data/subdomains.dat',
        'useragents': 'data/useragents.dat',
        'tmplist': 'tmp/list.tmp',
        'extensionlist': 'tmp/extensionlist.tmp',
        'ignore_extensionlist': 'tmp/ignore_extensionlist.tmp',
        'reports': 'reports/',
        'exceptions_log': 'syslog/exceptions.log',
    },
    'command': {
        'cvsupdate': '/usr/bin/git pull origin master',
        'cvslog': '/usr/bin/git log --oneline -n 1',
    },
    'examples': """

            Examples:
                python3 ./opendoor.py  --examples
                python3 ./opendoor.py  --update
                python3 ./opendoor.py  --version
                python3 ./opendoor.py  --docs
                python3 ./opendoor.py  --wizard
                python3 ./opendoor.py  --wizard /usr/local/projects/my.conf
                python3 ./opendoor.py --host "http://example.com"
                python3 ./opendoor.py --host "https://example.com" --port 8080
                python3 ./opendoor.py --host "http://example.com" --scan subdomains
                python3 ./opendoor.py --host "http://example.com" --threads 10
                python3 ./opendoor.py --host "http://example.com" -random-list --extensions php,html
                python3 ./opendoor.py --host "http://example.com" -random-list --ignore-extensions aspx,jsp
                python3 ./opendoor.py --host "http://example.com" --threads 10 --random-list
                python3 ./opendoor.py --host "http://example.com" --threads 10 --random-agent
                python3 ./opendoor.py --host "http://example.com" --threads 10 --tor
                python3 ./opendoor.py --host "http://example.com" --threads 10 --delay 10
                python3 ./opendoor.py --host "http://example.com" --threads 10 --prefix en/
                python3 ./opendoor.py --host "http://example.com" --threads 10 --delay 10 --timeout 10
                python3 ./opendoor.py --host "http://example.com"  --random-list --threads 10 --delay 10 --timeout 10
                python3 ./opendoor.py --host "http://example.com" --threads 10 --delay 10 --timeout 10 --debug 1
                python3 ./opendoor.py --host "http://example.com" --threads 10 --debug 1 --reports std,txt
                python3 ./opendoor.py --host "http://example.com" --debug 1 --reports std,txt --reports-dir /reports
                python3 ./opendoor.py --host "http://example.com" --threads 10 --debug 1 --extensions php,html
            """, 'banner': """
############################################################
#                                                          #
#   _____  ____  ____  _  _    ____   _____  _____  ____   #
#  (  _  )(  _ \( ___)( \( )  (  _ \ (  _  )(  _  )(  _ \  #
#   )(_)(  )___/ )__)  )  (    )(_) ) )(_)(  )(_)(  )   /  #
#  (_____)(__)  (____)(_)\_)  (____/ (_____)(_____)(_)\_)  #
#                                                          #
#  {0}\t\t                           #
#  {1}\t\t                           #
#  {2}\t\t\t                   #
#  {3}\t\t\t                           #
#  {4}                     #
############################################################""", 'version': """

{0}: {1} -> {2}
{3}
{4}
============================================================""", 'update': """

{status}
============================================================"""
}



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import unittest
from src.lib import browser, BrowserError
from src.lib.browser.threadpool import ThreadPool
from src.lib.browser.config import Config
from src.lib.browser.debug import Debug
from src.lib.reader.reader import Reader
from src.lib.tpl.tpl import Tpl
from src.core.http.response import Response
import os
from configparser import RawConfigParser
from ddt import ddt, data
from src.core import filesystem, helper


@ddt
class TestBrowser(unittest.TestCase):
    """TestBrowser class"""

    THREADS = 1

    @property
    def __configuration(self):
        test_config = filesystem.getabsname(os.path.join('tests', 'data', 'setup-scan.cfg'))
        config = RawConfigParser()
        config.read(test_config)
        return config

    def setUp(self):
        self.__pool = ThreadPool(num_threads=self.THREADS, total_items=10, timeout=0)

    def tearDown(self):
        del self.__pool

    def __browser_configuration(self, params):
        test_config = Config(params)
        return test_config

    def __browser_init(self, params):
        br = browser(params)
        return br

    def test_init(self):
        """ Browser.init() test """

        br = self.__browser_init({'host' : 'test.local', 'port' : 80})
        __client = getattr(br, '_Browser__client')
        __config = getattr(br, '_Browser__config')
        __debug = getattr(br, '_Browser__debug')
        __result = getattr(br, '_Browser__result')
        __reader = getattr(br, '_Browser__reader')
        __pool = getattr(br, '_Browser__pool')
        __response = getattr(br, '_Browser__response')

        self.assertIs(__client, None)
        self.assertTrue(isinstance(__config, Config))
        self.assertTrue(isinstance(__debug, Debug))
        self.assertTrue(isinstance(__result, dict))
        self.assertTrue(isinstance(__reader, Reader))
        self.assertTrue(isinstance(__pool, ThreadPool))
        self.assertTrue(isinstance(__response, Response))
    
    def test_init_exception(self):
        """ Browser.init() exception test """
        
        with self.assertRaises(BrowserError) as context:
            self.__browser_init({'host' : 'test.local', 'port' : 80, 'wordlist' : '/wrong'})
        self.assertTrue(BrowserError == context.expected)


    # @data(
    #         Config({'host': 'example.com', 'port' : 80, 'debug': 1}),
    #         Config({'host': 'example.com', 'scheme' : 'http://', 'port' : 80, 'debug': 1}),
    #         Config({'host': 'example.com', 'port': 80, 'debug': 2}),
    #         Config({'host': 'example.com', 'port': 80, 'debug': 3}),
    #         Config({'host': 'example.com', 'port': 80, 'debug': 3, 'indexof': True}),
    #         Config({'host': 'example.com', 'port': 80, 'debug': 3, 'random_agent': True}),
    #         Config({'host': 'example.com', 'port': 80, 'debug': 3, 'random_agent': True, 'random-list': True}),
    #         Config({'host': 'example.com', 'port': 80, 'debug': 3, 'accept-cookies' : True, 'scan' : 'directories'}),
    #         Config({'host': 'example.com', 'port': 80, 'debug': 3, 'scan' : 'subdomains'}),
    #         Config({'host': 'example.com', 'port': 80, 'debug': 3, 'threads' : 2, 'delay' : 1, 'timeout' : 10, 'request' : 3}),
    # )
    # def test_http_scan(self, config):
    #     """ Browser.scan() http test """
    #
    #     br = browser.__new__(browser)
    #     reader = Reader(browser_config={
    #             'list': config.scan,
    #             'torlist': config.torlist,
    #             'use_random': config.is_random_list,
    #             'is_external_wordlist': config.is_external_wordlist,
    #             'is_standalone_proxy': config.is_standalone_proxy,
    #             'is_external_torlist': config.is_external_torlist,
    #             'prefix': config.prefix
    #         })
    #     result = {}
    #     result['total'] = helper.counter()
    #     result['items'] = helper.list()
    #
    #     setattr(reader, '_Reader__config', self.__configuration)
    #     setattr(br, '_Browser__debug', Debug(config))
    #     setattr(br, '_Browser__pool', self.__pool)
    #     setattr(br, '_Browser__config', config)
    #     setattr(br, '_Browser__reader', reader)
    #     setattr(br, '_Browser__response', Response(config=config, debug=Debug(config), tpl=Tpl))
    #     setattr(br, '_Browser__result', result)
    #     self.assertIs(br.scan(), None)

    @data(
            Config({'host': 'http://example.com', 'port': 80, 'debug': 3, 'torlist':'/failed'}),
    )
    def test_browser_error(self, config):
        """ Browser.scan() exception test """

        br = browser.__new__(browser)
        with self.assertRaises(BrowserError) as context:
            reader = Reader(browser_config={
                'list': config.scan,
                'torlist': config.torlist,
                'use_random': config.is_random_list,
                'is_external_wordlist': config.is_external_wordlist,
                'is_standalone_proxy': config.is_standalone_proxy,
                'is_external_torlist': config.is_external_torlist,
                'prefix': config.prefix
            })
            result = {}
            result['total'] = helper.counter()
            result['items'] = helper.list()

            setattr(reader, '_Reader__config', self.__configuration)
            setattr(br, '_Browser__debug', Debug(config))
            setattr(br, '_Browser__pool', self.__pool)
            setattr(br, '_Browser__config', config)
            setattr(br, '_Browser__reader', reader)
            setattr(br, '_Browser__response', Response(config=config, debug=Debug(config), tpl=Tpl))
            setattr(br, '_Browser__result', result)
            br.scan()
        self.assertTrue(BrowserError == context.expected)

    @data({'reports' : 'std', 'host' : 'example.com', 'port' : 80})
    def test_ping(self, params):
        """ Browser.ping() test """

        br = browser.__new__(browser)
        setattr(br, '_Browser__config', self.__browser_configuration(params))
        self.assertIs(br.ping(), None)

    @data({'reports' : 'std', 'host' : 'test.local'})
    def test_done(self, params):
        """ Browser.done() test """

        br = browser.__new__(browser)
        result = {}

        result['total'] = helper.counter()
        result['items'] = helper.list()
        setattr(br, '_Browser__result', result)
        setattr(br, '_Browser__pool', self.__pool)
        setattr(br, '_Browser__pool.size', 0)
        setattr(br, '_Browser__config', self.__browser_configuration(params))
        self.assertIs(br.done(), None)

    @data({'reports' : 'raisesexc', 'host' : 'test.local'})
    def test_done_exception(self, params):
        """ Browser.done() exception test """

        br = browser.__new__(browser)
        result = {}

        result['total'] = helper.counter()
        result['items'] = helper.list()
        setattr(br, '_Browser__result', result)
        setattr(br, '_Browser__pool', self.__pool)
        setattr(br, '_Browser__pool.size', 0)
        setattr(br, '_Browser__config', self.__browser_configuration(params))

        with self.assertRaises(BrowserError) as context:
            br.done()
        self.assertTrue('raisesexc' in str(context.exception))
        self.assertTrue(BrowserError == context.expected)


if __name__ == "__main__":
    unittest.main()


# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from tabulate import tabulate

from .provider import PluginProvider
from src.core import sys


class StdReportPlugin(PluginProvider):
    """ StdReportPlugin class"""

    def __init__(self, target, data, directory=None):
        """
        PluginProvider constructor
        :param str target: target host
        :param dict data: result set
        """

        PluginProvider.__init__(self, target, data)
        self.directory = directory

    def process(self):
        """
        Process data
        :return: str
        """

        data = self._data.get('total').items()
        title = 'Statistics ({0})'.format(self._target)
        sys.writeln(tabulate(data, headers=[title, 'Summary'], tablefmt="psql"))



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from src.core import CoreSystemError, FileSystemError
from src.core import CoreConfig
from src.core import filesystem
from src.core import helper
from src.core import process
from src.core import sys
# noinspection PyPep8Naming
from src.lib.tpl import Tpl as tpl
from .exceptions import PackageError


class Package(object):
    """Package class"""

    remote_version = None

    @staticmethod
    def check_interpreter():
        """
        Get interpreter version
        :return: dict or bool
        """

        versions = CoreConfig.get('info').get('required_versions')
        actual_version = sys.version()
        target_compare = (actual_version == versions.get('minor') or actual_version == versions.get('major'))
        relative_compare = (helper.is_less(versions.get('minor'), actual_version) is True or
                            helper.is_more(actual_version, versions.get('major')) is True)

        if target_compare or relative_compare:
            return True
        else:
            return {
                'status': False,
                'actual': actual_version, 'expected': versions.get('minor') + ' -> ' + versions.get('major')
            }

    @staticmethod
    def examples():
        """
        Load examples of usage
        :return: str
        """

        examples = CoreConfig.get('examples')
        return examples

    @staticmethod
    def banner():
        """
        Load application banner

        :raise PackageError
        :return: str
        """

        try:

            banner = CoreConfig.get('banner').format(
                tpl.line('Directories: {0}'.format(Package.__directories_count()), color='blue'),
                tpl.line('Subdomains: {0}'.format(Package.__subdomains_count()), color='blue'),
                tpl.line('Browsers: {0}'.format(Package.__browsers_count()), color='blue'),
                tpl.line('Proxies: {0}'.format(Package.__proxies_count()), color='blue'),
                tpl.line(Package.__license(), color='blue'))

            return banner

        except (FileSystemError, CoreSystemError, PackageError) as error:
            raise PackageError(error)

    @staticmethod
    def version():
        """
        Load application version
        :raise PackageError
        :return: str
        """

        try:

            version = CoreConfig.get('version').format(Package.__app_name(), Package.__current_version(),
                                                       Package.__remote_version(), Package.__repo(),
                                                       Package.__license())
            return version

        except (FileSystemError, CoreSystemError, PackageError) as error:
            raise PackageError(error)

    @staticmethod
    def wizard(config):
        """
        Get application wizard (read from config)
        :param str config: configuration file
        :raise PackageError
        :return: str
        """

        try:
            config = filesystem.readcfg(config)
            # noinspection PyProtectedMember
            params = dict(config._sections['general'])

            for key, val in params.items():
                params[key] = None if val == 'None' else val
                if None is not params[key]:
                    if params[key].isdigit():
                        params[key] = int(float(params[key]))
                    elif params[key] in ['True', 'False']:
                        if params[key] == 'True':
                            params[key] = True
                        else:
                            params[key] = False
                    else:
                        params[key] = params[key].strip()
            return params

        except FileSystemError as error:
            raise PackageError(str(error))

    @staticmethod
    def docs():
        """
        Open documentation
        :raise PackageError
        :return: bool
        """
        docurl = CoreConfig.get('info').get('documentation')
        return helper.openbrowser(docurl)

    @staticmethod
    def update():
        """
        Check for update
        :raise PackageError
        :return: str
        """

        try:
            if False is sys().is_windows:
                status = process.execute(CoreConfig.get('command').get('cvsupdate'))
                upd_status = tpl.line(status, color='green')
                msg = CoreConfig.get('update').format(status=upd_status)
            else:
                msg = CoreConfig.get('update').format(status=tpl.line(key='upd_win_stat'))
            return msg
        except (AttributeError, CoreSystemError) as error:
            raise PackageError(error)

    @staticmethod
    def local_version():
        """
        Get application local version
        :raise PackageError
        :return: str
        """

        try:
            version = CoreConfig.get('info').get('version')
            return version
        except FileSystemError as error:
            raise PackageError(str(error))

    @staticmethod
    def __app_name():
        """
        Get application name
        :raise PackageError
        :return: str
        """

        try:
            name = CoreConfig.get('info').get('name')
            return name
        except FileSystemError as error:
            raise PackageError(str(error))

    @staticmethod
    def __remote_version():
        """
        Get application remote version
        :raise PackageError
        :return: str
        """

        if None is Package.remote_version:

            try:
                request_uri = CoreConfig.get('info').get('remote_version')
                result = process.execute('curl -sb GET {uri}'.format(uri=request_uri))
                Package.remote_version = str(result, "utf-8")
                return Package.remote_version

            except (FileSystemError, CoreSystemError) as error:
                raise PackageError(error)
        else:
            return Package.remote_version

    @staticmethod
    def __current_version():
        """
        Get current application version
        :raise PackageError
        :return: str
        """

        try:
            local = Package.local_version()
            remote = Package.__remote_version()
            if True is helper.is_less(local, remote):
                current_version = tpl.line(local, color='red')
            else:
                current_version = tpl.line(local, color='green')
            return current_version

        except (FileSystemError, CoreSystemError, PackageError) as error:
            raise PackageError(error)

    @staticmethod
    def __repo():
        """
        Get application repository url
        :raise PackageError
        :return: str
        """

        try:
            repository = CoreConfig.get('info').get('repository')
            return repository
        except FileSystemError as error:
            raise PackageError(str(error))

    @staticmethod
    def __license():
        """
        Get application license
        :raise PackageError
        :return: str
        """

        try:
            license_info = CoreConfig.get('info').get('license')
            return license_info
        except FileSystemError as error:
            raise PackageError(str(error))

    @staticmethod
    def __directories_count():
        """
        Get number of directories in basic wordlist
        :raise PackageError
        :return: int
        """

        try:
            filename = CoreConfig.get('data').get('directories')
            count = filesystem.count_lines(filename)

            return count

        except FileSystemError as error:
            raise PackageError(str(error))

    @staticmethod
    def __subdomains_count():
        """
        Get the number of subdomains in basic wordlist
        :raise PackageError
        :return: int
        """

        try:
            filename = CoreConfig.get('data').get('subdomains')
            count = filesystem.count_lines(filename)

            return count

        except FileSystemError as error:
            raise PackageError(str(error))

    @staticmethod
    def __browsers_count():
        """
        Get the number of browsers in basic wordlist
        :raise PackageError
        :return: int
        """

        try:
            filename = CoreConfig.get('data').get('useragents')
            count = filesystem.count_lines(filename)

            return count

        except FileSystemError as error:
            raise PackageError(str(error))

    @staticmethod
    def __proxies_count():
        """
        Get number of proxies in basic wordlist
        :raise PackageError
        :return: int
        """

        try:
            filename = CoreConfig.get('data').get('proxies')
            count = filesystem.count_lines(filename)

            return count

        except FileSystemError as error:
            raise PackageError(str(error))



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from urllib3 import HTTPConnectionPool, PoolManager, Timeout
from urllib3.exceptions import MaxRetryError, ReadTimeoutError, ConnectTimeoutError, HostChangedError
from src.core import helper
from .exceptions import HttpRequestError
from .providers import DebugProvider
from .providers import RequestProvider


class HttpRequest(RequestProvider, DebugProvider):

    """HttpRequest class"""

    def __init__(self, config, debug, **kwargs):
        """
        HttpRequest instance
        :param src.lib.browser.config.Config config: global configurations
        :param DebugProvider debug: debugger
        """

        try:
            self.__tpl = kwargs.get('tpl')
            RequestProvider.__init__(self, config, agent_list=kwargs.get('agent_list'))
            self.__headers = self._headers
            self.__connection_header = 'default'
            if True is config.keep_alive:
                self.__connection_header = self._keep_alive
        except (TypeError, ValueError) as error:
            raise HttpRequestError(error)

        self.__cfg = config
        self.__debug = debug

        if self.__cfg.DEFAULT_SCAN == self.__cfg.scan:
            self.__pool = self.__http_pool()

    def __http_pool(self):
        """
        Create HTTP connection pool
        :raise HttpRequestError
        :return: urllib3.HTTPConnectionPool
        """

        try:
            pool = HTTPConnectionPool(self.__cfg.host,
                                      port=self.__cfg.port,
                                      maxsize=self.__cfg.threads,
                                      timeout=Timeout(connect=self.__cfg.timeout, read=self.__cfg.timeout),
                                      block=True)
            if self._HTTP_DBG_LEVEL <= self.__debug.level:
                self.__debug.debug_connection_pool('http_pool_start', pool, self.__connection_header)
            return pool
        except Exception as error:
            raise HttpRequestError(str(error))

    def request(self, url):
        """
        Client request HTTP
        :param str url: request uri
        :return: urllib3.HTTPResponse
        """

        self.__headers.update({'User-Agent': self._user_agent})
        if 'default' is not self.__connection_header:
            self.__headers.update({'Connection': self.__connection_header})
        if self._HTTP_DBG_LEVEL <= self.__debug.level:
            self.__debug.debug_request(self.__headers, url, self.__cfg.method)
        try:
            if self.__cfg.DEFAULT_SCAN == self.__cfg.scan:
                response = self.__pool.request(self.__cfg.method,
                                               helper.parse_url(url).path,
                                               headers=self.__headers,
                                               retries=self.__cfg.retries,
                                               assert_same_host=True,
                                               redirect=False)
                self.cookies_middleware(is_accept=self.__cfg.accept_cookies, response=response)
            else:
                response = PoolManager().request(self.__cfg.method, url,
                                                 headers=self.__headers,
                                                 retries=self.__cfg.retries,
                                                 assert_same_host=False,
                                                 redirect=False,
                                                 timeout=Timeout(connect=self.__cfg.timeout, read=self.__cfg.timeout))
            return response

        except MaxRetryError:
            if self.__cfg.DEFAULT_SCAN == self.__cfg.scan:
                self.__tpl.warning(key='max_retry_error', url=helper.parse_url(url).path)
            pass

        except HostChangedError as error:
            self.__tpl.warning(key='host_changed_error', details=error)
            pass

        except ReadTimeoutError:
            self.__tpl.warning(key='read_timeout_error', url=url)
            pass

        except ConnectTimeoutError:
            self.__tpl.warning(key='connection_timeout_error', url=url)
            pass



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from __future__ import absolute_import
import unittest
# noinspection PyCompatibility
import argparse
from ddt import ddt, data
from src.core.options import Options
from src.core.options.exceptions import OptionsError

@ddt
class TestOptions(unittest.TestCase):
    """TestOptions class"""

    @data(
            {'version': True, 'examples': False, 'update': None, 'docs' : None},
            {'examples': True, 'version': None, 'update': False, 'docs' : None},
            {'update': True , 'version': False, 'examples': False, 'docs' : None}
    )
    def test_get_arg_values_standalone(self, data):
        """ Arguments.get_arg_values() standalone call test """
    
        opt = Options.__new__(Options)
        args = argparse.Namespace()
        args.host = ''
        args.version = data.get('version')
        args.update = data.get('update')
        args.examples = data.get('examples')
        args.docs = data.get('docs')
        setattr(opt, '_Options__standalone', ["version", "update", "examples","docs"])
        setattr(opt, 'args', args)
        actual = opt.get_arg_values()
        self.assertTrue(isinstance(actual, dict))
        self.assertTrue(1 == len(actual))

    @data(
            {'host': 'example.com', 'port': 80},
            {'host': 'https://example.com', 'port': 223},
            {'host': 'example.com','scan': 'directories'},
            {'host': 'example.com','scan': 'subdomains'},
            {'host': 'example.com', 'proxy': '127.0.0.1'},
    )
    def test_get_arg_values(self, data):
        """ Arguments.get_arg_values() test """

        opt = Options.__new__(Options)
        args = argparse.Namespace()
        args.host = data.get('host')
        args.port = data.get('port')
        args.scan = data.get('scan')
        args.version = False
        args.update = False
        args.examples = False
        args.docs = False
        setattr(opt, '_Options__standalone', ["version", "update", "examples", "docs"])
        setattr(opt, 'args', args)
        actual = opt.get_arg_values()
        self.assertTrue(isinstance(actual, dict))

    def test_get_arg_values_exception2(self):
        """ Arguments.get_arg_values() exception2 test """

        opt = Options.__new__(Options)
        setattr(opt, '_Options__standalone', ["version", "update", "examples", "docs"])
        setattr(opt, 'args', {})
        with self.assertRaises(OptionsError) as context:
            opt.get_arg_values()
        self.assertTrue(OptionsError == context.expected)

    def test_get_arg_values_exception3(self):
        """ Arguments.get_arg_values() exception3 test """

        opt = Options.__new__(Options)
        args = argparse.Namespace()
        args.host = ''
        args.version = False
        args.update = False
        args.examples = False
        args.docs = False
        setattr(opt, '_Options__standalone', ["version", "update", "examples", "docs"])
        setattr(opt, 'args', args)
        with self.assertRaises(OptionsError) as context:
            opt.get_arg_values()
        self.assertTrue(OptionsError == context.expected)

if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from configparser import RawConfigParser, ParsingError, NoOptionError
import errno
import os
import random
import re

from .exceptions import FileSystemError


class FileSystem(object):

    """FileSystem class"""

    sep = os.sep

    @staticmethod
    def is_exist(directory, filename):
        """
        Check if dir-file is exist
        :param str directory: directory
        :param str filename: filename
        :return: bool
        """

        path = os.path.join(directory, filename)
        if not os.path.exists(path):
            return False
        else:
            return True

    @staticmethod
    def makedir(directory, mode=0o777):
        """
        Create new directory

        :param str directory: directory
        :param int mode: directory permission
        :raise: FileSystemError
        :return: str
        """

        if not os.path.exists(directory) or not os.access(directory,  os.W_OK):

            try:
                directory = os.path.join(directory)
                directory += '/'
                os.makedirs(directory, mode=mode)
            except OSError:
                try:
                    directory = os.path.join(os.path.expanduser('~'), directory)
                    os.makedirs(directory, mode=mode)
                except OSError as error:
                    if error.errno != errno.EEXIST:
                        raise FileSystemError("Cannot create directory `{0}`. Reason: {1}".format(directory, error))
        return directory

    @staticmethod
    def getabsname(filename):
        """
        Get absolute file path
        :param str filename: directory
        :return: str
        """

        filename = os.path.join(filename)
        return os.path.abspath(filename)

    @staticmethod
    def get_extension(line):
        """
        Get extension from line
        :param str line: input string
        :return: str
        """

        ext = os.path.splitext(line)[-1]
        return ext

    @staticmethod
    def has_extension(line):
        """
        Check line for extension
        :param str line: input string
        :return: bool
        """

        ext = FileSystem.get_extension(line)
        return True if 0 < len(ext) else False

    @staticmethod
    def filter_file_lines(dirlist, pattern):
        """
        Filter lines by regex pattern

        :param list dirlist:
        :param str pattern:
        :return: list
        """

        r = re.compile(pattern)
        newlist = filter(r.match, dirlist)
        filteredlist = list(newlist)
        return filteredlist

    @staticmethod
    def clear(directory, extension=''):
        """
        Clear directory
        :param str directory: os directory
        :param str extension: extension
        :raise: FileSystemError
        :return: None
        """

        if True is os.path.exists(directory):
            try:
                for files in os.listdir(directory):
                    filename = os.path.join(directory, files)
                    file_extension = os.path.splitext(filename)[1]
                    if extension == file_extension:
                        os.remove(os.path.join(directory, files))
            except IOError as error:
                raise FileSystemError(error.strerror)
        else:
            raise FileSystemError("The directory {0} you want to clear is not exist".format(directory))

    @staticmethod
    def makefile(filename):
        """
        Create a new file with context
        :param str filename: input filename
        :raise: FileSystemError
        :return: str
        """

        filename = os.path.join(filename)
        if False is os.path.exists(filename) or False is os.access(filename, os.R_OK):
            try:
                directory = FileSystem.makedir(os.path.dirname(filename))
                abs_filename = os.path.join(directory, os.path.basename(filename))
                open(abs_filename, 'w')
                return abs_filename
            except IOError as error:
                raise FileSystemError(error.strerror)
        else:
            return filename

    @staticmethod
    def shuffle(target, output, total):
        """
        Shuffle data in file
        :param str target: target file
        :param str output: suffled file
        :param int total: total lines
        :return: bool
        """

        try:
            i_f = open(target)
            o_f = open(output, 'wb')
            counter = sum(1 for _ in i_f)
            order = list(range(counter))
            random.shuffle(order)
            while order:
                current_lines = {}
                current_lines_count = 0
                current_chunk = order[:total]
                current_chunk_dict = dict((x, 1) for x in current_chunk)
                current_chunk_length = len(current_chunk)
                order = order[total:]
                i_f.seek(0)
                count = 0

                for line in i_f:
                    if count in current_chunk_dict:
                        current_lines[count] = line
                        current_lines_count += 1
                        if current_lines_count == current_chunk_length:
                            break
                    count += 1

                for node in current_chunk:
                    o_f.write(current_lines[node].encode())

        except IOError as error:
            raise FileSystemError(error.strerror)

    @staticmethod
    def readline(filename, handler, handler_params, loader):
        """
        Read txt file line by line
        :param str filename: source file name
        :param func handler: url handler
        :param func handler_params: url handler parameters
        :param func loader: browser
        :raise FileSystemError
        :return: str
        """

        filepath = os.path.join(filename)
        if not os.path.isfile(filepath):
            filepath = os.path.join(os.path.expanduser('~'), filepath)
            if not os.path.isfile(filepath):
                raise FileSystemError("{0} is not a file ".format(filepath))
        if not os.access(filepath, os.R_OK):
            filepath = os.path.join(os.path.expanduser('~'), filepath)
            if not os.access(filepath, os.R_OK):
                raise FileSystemError("Configuration file {0} can not be read. Setup chmod 0644".format(filepath))

        lines = []
        with open(filepath) as f_handler:
            for line in f_handler:
                lines.append(handler(line, handler_params))
            loader(lines)

    @staticmethod
    def read(filename):
        """
        Read .txt file
        :param str filename: input filename
        :raise FileSystemError
        :return: list
        """

        filepath = os.path.join(filename)

        if not os.path.isfile(filepath):
            raise FileSystemError("{0} is not a file ".format(filename))
        if not os.access(filepath, os.R_OK):
            raise FileSystemError("Configuration file {0} can not be read. Setup chmod 0644".format(filepath))

        with open(filepath, 'r') as f_handler:
            data = f_handler.readlines()
        return data

    @staticmethod
    def count_lines(filename):
        """
        Count lines in .txt file
        :param str filename: input filename
        :raise FileSystemError
        :return: list
        """

        filepath = os.path.join(filename)

        if not os.path.isfile(filepath):
            raise FileSystemError("{0} is not a file ".format(filename))
        if not os.access(filepath, os.R_OK):
            raise FileSystemError("Configuration file {0} can not be read. Setup chmod 0644".format(filepath))
        with open(filepath, 'r') as f_handler:
            for count, line in enumerate(f_handler):
                pass
        return count + 1

    @staticmethod
    def readcfg(filename):
        """
        Read .cfg file
        :param str filename: input filename
        :raise FileSystemError
        :return: configparser.RawConfigParser
        """

        expression = '^([\/a-z].*?opendoor.*?)\/'
        find_dir = re.search(expression, __file__, re.IGNORECASE)
        if None is not find_dir:
            os.chdir(find_dir.group())
        filepath = os.path.join(os.path.sep, os.getcwd(), filename)

        if not os.path.isfile(filepath):
            raise FileSystemError("{0} is not a file ".format(filepath))
        if not os.access(filepath, os.R_OK):
            raise FileSystemError("Configuration file {0} can not be read. Setup chmod 0644".format(filepath))

        try:

            config = RawConfigParser()
            config.read(filepath)
            return config

        except (ParsingError, NoOptionError) as error:
            raise FileSystemError(error)

    @staticmethod
    def writelist(filename, data, separator=''):
        """
        Write the list to file
        :param str filename: input filename
        :param list data: record data
        :param str separator: line separator
        :raise FileSystemError
        :return: None
        """

        filepath = os.path.join(filename)
        if not os.path.isfile(filepath):
            raise FileSystemError("{0} is not a file ".format(filename))
        if not os.access(filepath, os.W_OK):
            raise FileSystemError("Targeting file {0} is not writable. Please, check access".format(filepath))

        with open(filepath, "w") as f_handler:
            f_handler.write(separator.join(data))

    @staticmethod
    def human_size(size, precision=2):
        """
        Humanize accepted bytes
        :param int size: bytes
        :param int precision: delimiter
        :return:
        """

        suffixes = ['B', 'KB', 'MB', 'GB', 'TB']
        suffix_index = 0
        size = int(size)
        while size > 1024 and suffix_index < 4:
            suffix_index += 1
            size /= 1024

        return "%.*f%s" % (precision, size, suffixes[suffix_index])



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from io import StringIO
from mock import patch
import unittest
from src.lib.browser.filter import Filter
from src.lib.browser.config import Config
from src.core.logger.logger import Logger


class TestBrowserFilter(unittest.TestCase):
    """TestBrowserFilter class"""
    
    def setUp(self):
        
        logger = Logger.log()
        for handler in logger.handlers:
            logger.removeHandler(handler)
            
        self.config = Config({
        'debug' : 1,
        'method' : 'HEAD',
        'indexof' : None,
        'random_agent': False,
        'random_list' : True,
        'threads' : 1000
    })
        
    def test_filter_threads_to_max(self):
        """ Filter.init() test max threads """
        
        with patch('sys.stdout', new=StringIO()):
            Filter(self.config, 300)
            self.assertEqual(self.config.threads, Config.DEFAULT_MAX_THREADS)
            
    def test_filter_threads_to_list_lines(self):
        """ Filter.init() test max lines"""

        with patch('sys.stdout', new=StringIO()):
            Filter(self.config, 10)
            self.assertEqual(self.config.threads, 10)


if __name__ == "__main__":
    unittest.main()


# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from .provider import PluginProvider
from src.core import CoreConfig
from src.core import filesystem, FileSystemError
from json2html.jsonconv import Json2Html


class HtmlReportPlugin(PluginProvider):
    """ HtmlReportPlugin class"""

    PLUGIN_NAME = 'HtmlReport'
    EXTENSION_SET = '.html'

    def __init__(self, target, data, directory=None):
        """
        PluginProvider constructor
        :param str target: target host
        :param dict data: result set
        :param str directory: custom directory
        """

        PluginProvider.__init__(self, target, data)

        try:

            if None is directory:
                directory = CoreConfig.get('data').get('reports')
            self.__target_dir = filesystem.makedir("".join((directory, self._target)))
        except FileSystemError as error:
            raise Exception(error)

    def process(self):
        """
        Process data
        :return: str
        """

        try:
            filesystem.clear(self.__target_dir, extension=self.EXTENSION_SET)
            resultset = Json2Html().convert(json=self._data, table_attributes='border="1" cellpadding="2"')
            self.record(self.__target_dir, self._target, resultset)
        except FileSystemError as error:
            raise Exception(error)



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import unittest
from src.core import sys
from src.lib.package import Package, PackageError
from src.core.filesystem import FileSystem
from src.core.logger.logger import Logger


class TestPackage(unittest.TestCase):
    """TestPackage class"""
    
    def tearDown(self):
        logger = Logger.log()

        for handler in logger.handlers:
            logger.removeHandler(handler)
            
    def test_check_interpreter(self):
        """ Package.check_interpreter() test """

        expected = Package.check_interpreter()
        self.assertTrue(expected)

    def test_examples(self):
        """ Package.examples() test """

        expected = Package.examples()
        self.assertIsNotNone(expected)
        self.assertIs(type(expected), str)

    # def test_banner(self):
    #     """ Package.banner() test """
    #
    #     Config.params['cfg'] = 'setup.cfg'
    #     expected = Package.banner()
    #     self.assertIsNotNone(expected)
    #     self.assertIs(type(expected), str)
    #
    # def test_version(self):
    #     """ Package.version() test """
    #
    #     Config.params['cfg'] = 'setup.cfg'
    #     expected = Package.version()
    #     self.assertIsNotNone(expected)
    #     self.assertIs(type(expected), str)
    #
    # def test_update_unix(self):
    #     """ Package.update() unix test """
    #
    #     Config.params['cfg'] = 'setup.cfg'
    #     Config.params['update'] = '{status}'
    #     expected = Package.update()
    #     self.assertIsNotNone(expected)
    #     self.assertIs(type(expected), str)
    #
    # def test_update_windows(self):
    #     """ Package.update() test """
    #
    #     Config.params['cfg'] = 'setup.cfg'
    #     Config.params['update'] = '{status}'
    #     setattr(sys, 'is_windows', True)
    #     expected = Package.update()
    #     self.assertIsNotNone(expected)

    # def test_local_version(self):
    #     """ Package.local_version() test """
    #
    #     Config.params['cfg'] = 'setup.cfg'
    #     actual = FileSystem.readcfg('setup.cfg').get('info', 'version')
    #     expected = Package.local_version()
    #     self.assertEqual(actual, expected)
    #
    # def test_local_version_exception(self):
    #     """ Package.local_version() exception test """
    #
    #     Config.params['cfg'] = 'wrong.cfg'
    #     with self.assertRaises(PackageError) as context:
    #         Package.local_version()
    #         self.assertTrue(PackageError == context.expected)
    #
    # def test_update_exception(self):
    #     """ Package.update() exception test """
    #     Config.params['cvsupdate'] = 'wrongcvs'
    #     with self.assertRaises(PackageError) as context:
    #         Package.update()
    #         self.assertTrue(PackageError == context.expected)
    #     Config.params['cvsupdate'] = '/usr/bin/git pull origin master'
    #
    # def test_version_exception(self):
    #     """ Package.version() exception test """
    #
    #     Config.params['cfg'] = 'wrong.cfg'
    #     with self.assertRaises(PackageError) as context:
    #         Package.version()
    #         self.assertTrue(PackageError == context.expected)
    #
    # def test_banner_exception(self):
    #     """ Package.banner() exception test """
    #
    #     Config.params['cfg'] = 'wrong.cfg'
    #     with self.assertRaises(PackageError) as context:
    #         Package.banner()
    #         self.assertTrue(PackageError == context.expected)
            
if __name__ == "__main__":
    unittest.main()



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""


class Config(object):

    """Config class"""

    DEFAULT_SOCKET_TIMEOUT = 10
    DEFAULT_MIN_THREADS = 1
    DEFAULT_MAX_THREADS = 25
    DEFAULT_DEBUG_LEVEL = 1
    DEFAULT_REPORT = 'std'
    DEFAULT_SCAN = 'directories'
    SUBDOMAINS_SCAN = 'subdomains'
    DEFAULT_SCHEME = 'http://'
    DEFAULT_HTTP_PORT = 80
    DEFAULT_SSL_PORT = 443
    DEFAULT_HTTP_METHOD = 'HEAD'
    DEFAULT_USER_AGENT = 'PostmanRuntime/7.29.0'

    def __init__(self, params):
        """
        Read filtered input params
        :param dict params: input cli arguments
        """

        self._scan = params.get('scan')
        self._scheme = self.DEFAULT_SCHEME if params.get('scheme') is None else params.get('scheme')
        self._ssl = params.get('ssl')
        self._host = params.get('host')
        self._proxy = '' if params.get('proxy') is None else params.get('proxy')
        self._accept_cookies = False if params.get('accept_cookies') is None else True
        self._keep_alive = False if params.get('keep_alive') is None else True
        self._port = params.get('port')
        self._wordlist = params.get('wordlist')
        self._reports_dir = params.get('reports_dir')
        self._prefix = "" if params.get('prefix') is None else params.get('prefix')
        self._reports = params.get('reports')
        self._extensions = params.get('extensions')
        self._ignore_extensions = params.get('ignore_extensions')
        self._retries = False if params.get('retries') is None else params.get('retries')
        self._method = params.get('method')
        self._delay = params.get('delay')
        self._timeout = params.get('timeout')
        self._debug = self.DEFAULT_DEBUG_LEVEL if params.get('debug') is None else params.get('debug')
        self._is_tor = params.get('tor')
        self._torlist = '' if 'torlist' not in params else params.get('torlist')
        self._is_random_user_agent = params.get('random_agent')
        self._sniff = params.get('sniff')
        self._is_random_list = False if params.get('random_list') is None else True
        self._is_extension_filter = False if params.get('extensions') is None else True
        self._is_ignore_extension_filter = False if params.get('ignore_extensions') is None else True
        self._user_agent = self.DEFAULT_USER_AGENT
        self._threads = self.DEFAULT_MIN_THREADS if params.get('threads') is None else params.get('threads')

    @property
    def scan(self):
        """
        Scan property
        :return: str
        """
        
        return self.DEFAULT_SCAN if self._scan is None else self._scan

    @scan.setter
    def scan(self, value):
        """
        scan param setter
        :param str value:
        :return: None
        """
        self._scan = value

    @property
    def scheme(self):
        """
        Protocol property
        :return: str
        """

        return self._scheme

    @property
    def is_ssl(self):
        """
        If using ssl
        :return: bool
        """

        return self._ssl

    @property
    def prefix(self):
        """
        Paths prefix
        :return: str
        """

        return self._prefix.lstrip("/")

    @property
    def host(self):
        """
        Hostname property
        :return: str
        """

        return self._host

    @property
    def port(self):
        """
        Port property
        :return: int
        """
        
        if True is self._ssl and self._port is self.DEFAULT_HTTP_PORT:
            self._port = self.DEFAULT_SSL_PORT
        return self._port

    @property
    def method(self):
        """
        Scan method property
        :return: str
        """

        if True is self.is_sniff:
            if 1 == len(self.sniffers) and 'file' == self.sniffers[0]:
                return 'HEAD'
            return 'GET'
        return self.DEFAULT_HTTP_METHOD if self._method is None else self._method

    @property
    def delay(self):
        """
        Delay property
        :return: int
        """

        if None is self._delay:
            self._delay = 0
        elif 1 <= self._delay:
            self._delay = int(self._delay)

        return self._delay

    @property
    def timeout(self):
        """
        Timeout property
        :return: float
        """
        return self._timeout

    @timeout.setter
    def timeout(self, value):
        """
        Timeout param setter
        :param int value:
        :return: None
        @param value:
        @return:
        """

        self._timeout = float(value)
        return self._timeout

    @property
    def retries(self):
        """
        Retries property
        :return: int
        """

        return self._retries

    @property
    def debug(self):
        """
         Debug level property
         :return: int
         """

        return self._debug

    @property
    def proxy(self):
        """
        Standalone proxy property
        :return: bool
        """

        return self._proxy

    @property
    def is_proxy(self):
        """
        If proxy is available
        :return: bool
        """

        if True is self._is_tor:
            return True
        elif None is not self._torlist and 0 < len(self._torlist):
            return True
        elif None is not self._torlist and 0 < len(self._proxy):
            return True

        return False

    @property
    def is_random_user_agent(self):
        """
        If ua randomizing is available
        :return: bool
        """

        return self._is_random_user_agent

    @property
    def is_sniff(self):
        """
        If sniffers are available
        :return: bool
        """

        if None is not self._sniff:
            if False is isinstance(self._sniff, list):
                self._sniff = self._sniff.split(",")
            if 0 < len(self._sniff):
                return True
        return False

    @property
    def sniffers(self):
        """
        Get a sniffer
        :return: list
        """

        return self._sniff

    @property
    def is_random_list(self):
        """
        If scan lists randomize and are available
        :return: bool
        """

        return self._is_random_list

    @property
    def is_extension_filter(self):
        """
        If scan lists filtered by extensions
        :return: bool
        """

        return self._is_extension_filter

    @property
    def is_ignore_extension_filter(self):
        """
        If a scan list filtered by ignore extensions
        :return: bool
        """

        return self._is_ignore_extension_filter

    @property
    def is_standalone_proxy(self):
        """
        If standalone proxy is available
        :return: bool
        """

        if True is self.is_proxy and 0 < len(self._proxy):
            self._torlist = ''

            return True
        return False

    @property
    def is_internal_torlist(self):
        """
        If internal torlist is available
        :return: bool
        """

        if True is self._is_tor and 0 >= len(self._torlist):
            return True
        return False

    @property
    def is_external_torlist(self):
        """
        If external torlist is available
        :return: bool
        """

        if None is not self._torlist and 0 < len(self._torlist):
            return True
        return False

    @property
    def torlist(self):
        """
        Torlist property
        :return: bool
        """

        return self._torlist

    @property
    def is_external_wordlist(self):
        """
        If exteranl word list is available
        :return: bool
        """

        if None is self._wordlist:
            return False
        return True

    @property
    def is_external_reports_dir(self):
        """
        If exteranl reports directory selected
        :return: bool
        """

        if None is self._reports_dir:
            return False
        return True

    @property
    def reports_dir(self):
        """
        Get reports dir
        :return: str
        """

        return self._reports_dir

    @property
    def wordlist(self):
        """
        Get external wordlist
        :return: str
        """

        return self._wordlist

    @property
    def extensions(self):
        """
        Extensions resolver
        :return: list
        """

        extensions = None
        if None is not self._extensions:
            extensions = self._extensions.split(",")
        return extensions

    @property
    def ignore_extensions(self):
        """
        Extensions resolver
        :return: list
        """

        extensions = None
        if None is not self._ignore_extensions:
            extensions = self._ignore_extensions.split(",")
        return extensions

    @property
    def reports(self):
        """
        Reports resolver
        :return: list
        """

        reports = self._reports.split(",")
        if self.DEFAULT_REPORT not in reports:
            reports.append(self.DEFAULT_REPORT)
        return reports

    @property
    def user_agent(self):
        """
        User agent property
        :return: str
        """

        return self._user_agent

    def set_threads(self, threads):
        """
        Threads setter
        :param int threads: threads
        :return: int
        """

        self._threads = threads

    @property
    def threads(self):
        """
        Threads property
        :return: int
        """

        return self._threads

    @property
    def accept_cookies(self):
        """
        Accept cookies property
        :return: bool
        """

        return self._accept_cookies

    @property
    def keep_alive(self):
        """
        If connection keep-alive
        :return: bool
        """

        return self._keep_alive



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from setuptools import setup, find_packages
fVersion = open('VERSION').readline().rstrip()

NAME = 'opendoor'
VERSION = fVersion
DESCRIPTION = 'OWASP WEB Directory Scanner'
LONG_DESCRIPTION = open('README.md', 'r+', encoding='utf-8').read()
HOMEPAGE = 'https://github.com/stanislav-web/OpenDoor'
AUTHOR = 'Brain Storm Team',
AUTHOR_EMAIL = 'nomail@gmail.com',
MAINTAINER = 'Brain Storm Team',
FULL_NAME = 'Brain Storm'
DOWNLOAD_URL = 'https://github.com/stanislav-web/OpenDoor/archive/refs/heads/master.zip'
setup(name=NAME,

      # Versions should comply with PEP440.  For a discussion on single-sourcing
      # the version across setup.py and the project code, see
      # https://packaging.python.org/en/latest/single_source_version.html

      version=VERSION,

      description=DESCRIPTION,

      long_description=LONG_DESCRIPTION,

      long_description_content_type="text/markdown",

      # The project's main homepage.
      url=HOMEPAGE,

      # Author details
      author="".join(AUTHOR),

      fullname="".join(FULL_NAME),

      author_email="".join(AUTHOR_EMAIL),

      maintainer="".join(MAINTAINER),

      maintainer_email="".join(AUTHOR_EMAIL),

      # You can just specify the packages manually here if your project is
      # simple. Or you can use find_packages().
      zip_safe=False,
      packages=find_packages(),
      data_files=[('.', ['opendoor.conf']),
                  ('data', [
                      'data/directories.dat',
                      'data/ignored.dat',
                      'data/proxies.dat',
                      'data/subdomains.dat',
                      'data/useragents.dat',
                  ])
                  ],
      include_package_data=True,

      script_name='opendoor.py',
      # Choose your license
      license='GPL',
      # Unittests suite directory
      test_suite='tests',

      # What does your project relate to?
      keywords=[
          'owasp scanner',
          'directory scanner',
          'access directory scanner',
          'fuzzer',
          'auth scanner',
          'dir search',
          'dirmap'
      ],

      download_url=DOWNLOAD_URL,

      # To provide executable scripts, use entry points in preference to the
      # "scripts" keyword. Entry points provide cross-platform support and allow
      # pip to create the appropriate form of executable for the target platform.
      entry_points={'console_scripts': [
          'opendoor=src:main',
          'coveralls = coveralls.cli:main'
      ]},

      install_requires=[line.rstrip('\n') for line in open('requirements.txt')],
      tests_require=[line.rstrip('\n') for line in open('requirements-dev.txt')],

      platforms=['any'],
      # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
      # How mature is this project? Common values are
      classifiers=[
          #   3 - Alpha
          #   4 - Beta
          #   5 - Production/Stable
          'Development Status :: 5 - Production/Stable',

          # Language
          'Natural Language :: English',

          # Indicate who your project is intended for
          'Intended Audience :: Developers',

          # OS, which support this package
          'Operating System :: MacOS',
          'Operating System :: Unix',

          # Pick your license as you wish (should match "license" above)
          'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

          # Specify the Python versions you support here. In particular, ensure
          # that you indicate whether you support Python 2, Python 3 or both.

          'Programming Language :: Python',
          'Programming Language :: Python :: 3',

          # Specify the additional categories
          'Topic :: Utilities',
          'Topic :: Internet :: WWW/HTTP :: Site Management :: Link Checking',
          'Topic :: Internet :: WWW/HTTP :: Indexing/Search',
          'Topic :: Internet :: WWW/HTTP :: WSGI :: Application',
      ])



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import sys
import platform
import os


class Output(object):

    """Output class"""

    __is_windows = None

    @staticmethod
    def exit(msg):
        """
        Abort session
        :param str msg: input message
        :return: None
        """

        sys.exit(msg)

    @staticmethod
    def writels(msg, flush=True):
        """
        Write to stdout on one line dynamically
        :param str msg: input message
        :param bool flush: force clear line
        :return: None
        """

        sys.stdout.write("\r\x1b[K" + msg.__str__())
        if True is flush:
            sys.stdout.flush()

    @staticmethod
    def writeln(msg):
        """
        Write new line
        :param str msg: input message
        :return: None
        """

        sys.stdout.write('{0}\n'.format(msg))

    @staticmethod
    def version():
        """
        Interpreter version
        :return: string
        """

        version = platform.python_version().split(".")
        return "{0}.{1}".format(version[0], version[1])

    @property
    def is_windows(self):
        """
        Check for windows signature
        :return: bool
        """

        if None is Output.__is_windows:
            Output.__is_windows = True if sys.platform.startswith('win') or os.name == 'nt' else False
        return Output.__is_windows



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""
import unittest
from src.lib.browser.config import Config


class TestBrowserConfig(unittest.TestCase):
    """TestBrowserConfig class"""

    def test_config_properties(self):
        """ Config.init() """
        
        empty_config = Config({})
        self.assertIs(type(Config.scan), property)
        self.assertTrue(str(Config.DEFAULT_SCAN) is str(empty_config.scan))
        
        self.assertIs(type(Config.scheme), property)
        self.assertTrue('http://' is str(Config({'scheme' : 'http://'}).scheme))
        
        self.assertIs(type(Config.is_ssl), property)
        self.assertFalse(empty_config.is_ssl)

        self.assertIs(type(Config.prefix), property)
        self.assertTrue('test/' is str(Config({'prefix': 'test/'}).prefix))

        self.assertIs(type(Config.host), property)
        self.assertTrue('example.com' is str(Config({'host': 'example.com'}).host))

        self.assertIs(type(Config.port), property)
        self.assertTrue(80 is Config({'port': 80}).port)
        
        self.assertIs(type(Config.method), property)
        self.assertTrue(str(Config.DEFAULT_HTTP_METHOD) is str(Config({'method' : 'HEAD'}).method))

        self.assertIs(type(Config.delay), property)
        self.assertTrue(1 is Config({'delay' : 1}).delay)

        self.assertIs(type(Config.timeout), property)

        self.assertAlmostEqual(Config.DEFAULT_SOCKET_TIMEOUT, Config({'timeout' : 10}).timeout)

        self.assertIs(type(Config.retries), property)
        self.assertTrue(3 is Config({'retries' : 3}).retries)

        self.assertIs(type(Config.debug), property)
        self.assertTrue(1 is Config({'debug' : 1}).debug)
        
        self.assertIs(type(Config.proxy), property)
        self.assertTrue('' is empty_config.proxy)
        
        self.assertIs(type(Config.is_proxy), property)
        self.assertFalse(empty_config.is_proxy)

        self.assertIs(type(Config.is_random_user_agent), property)
        self.assertFalse(empty_config.is_random_user_agent)

        self.assertIs(type(Config.is_random_list), property)
        self.assertFalse(empty_config.is_random_list)
        
        self.assertIs(type(Config.is_standalone_proxy), property)
        self.assertFalse(empty_config.is_standalone_proxy)

        self.assertIs(type(Config.is_internal_torlist), property)
        self.assertFalse(empty_config.is_internal_torlist)

        self.assertIs(type(Config.is_external_torlist), property)
        self.assertFalse(empty_config.is_external_torlist)

        self.assertIs(type(Config.is_external_wordlist), property)
        self.assertFalse(empty_config.is_external_wordlist)
        
        self.assertIs(type(Config.torlist), property)
        self.assertTrue('' is empty_config.torlist)

        self.assertIs(type(Config.reports), property)
        self.assertIs(type(Config({'reports' : 'std'}).reports), list)
        self.assertTrue(Config.DEFAULT_REPORT in Config({'reports' : 'std'}).reports)
        
        self.assertIs(type(Config.user_agent), property)
        self.assertTrue(Config.DEFAULT_USER_AGENT is empty_config.user_agent)

        empty_config.set_threads(Config.DEFAULT_MIN_THREADS)
        self.assertIs(type(Config.threads), property)
        self.assertTrue(empty_config.threads == Config.DEFAULT_MIN_THREADS)
        
        self.assertIs(type(Config.accept_cookies), property)
        self.assertFalse(empty_config.accept_cookies)

if __name__ == "__main__":
    unittest.main()


# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import logging
import re

from .colorize import ColorizingStreamHandler
from src.core.system import Term


class RainbowLoggingHandler(ColorizingStreamHandler):

    """ Class RainbowLoggingHandler """

    # Define color for message payload
    level_map = {
        logging.DEBUG: ('cyan', False),
        logging.INFO: ('white', False),
        logging.WARNING: ('yellow', False),
        logging.ERROR: ('red', True),
        logging.CRITICAL: ('red', True)
    }

    date_format = "%H:%M:%S"

    #: How many characters reserve to function name logging
    who_padding = 7

    def get_color(self, fg=None, bold=False):
        """
        Construct a terminal color code
        :param str fg: Symbolic name of foreground color
        :param bool bold: Brightness bit
        """

        params = []

        if fg in self.color_map:
            params.append(str(self.color_map[fg] + 30))
        if bold:
            params.append('1')

        color_code = ''.join((self.csi, ';'.join(params), 'm'))

        return color_code

    def colorize(self, record):
        """
        Get a special format string with ASCII color codes
        :param dict|None record: logging record
        :return: str
        """

        # Dynamic message color based on logging level
        if not hasattr(record, 'levelno'):
            record.levelno = 20
        fg, bold = self.level_map[record.levelno]

        template = [
            "[", self.get_color("black"),
            "%(asctime)s", self.reset, "] ",
            "",
            "",
            "%(padded_who)s", self.reset, " ", self.get_color(fg, bold), "%(message)s", self.reset]

        format_string = "".join(template)

        who = [self.get_color("green"), getattr(record, "funcName", ""), self.get_color("black", True), ":",
               self.get_color("cyan")]

        who = "".join(who)
        # We need to calculate padding length manualy
        # as color codes mess up string length based calcs
        unformatted_who = getattr(record, "funcName", "")

        if len(unformatted_who) < self.who_padding:
            spaces = " " * (self.who_padding - len(unformatted_who))
        else:
            spaces = ""

        record.padded_who = who + spaces

        formatter = logging.Formatter(format_string, self.date_format)
        output = formatter.format(record)
        # Clean cache so the color codes of traceback don't leak to other formatters
        record.ext_text = None
        # noinspection PyUnresolvedReferences
        width = int(Term.terminal_size['width'])
        pure_length = self.__pure_line_len(output)
        length = width - pure_length
        if record.levelno != logging.DEBUG:
            if pure_length > width and record.levelno != logging.ERROR:
                output = (output[:width] + '...')

        end = (' ' * length)[:length]
        return output + end

    @classmethod
    def __pure_line_len(cls, string):
        """
        Get pure line
        :param str string: input string
        :return: str
        """

        ansi_escape = re.compile(r'\x1b[^m]*m')
        return len(ansi_escape.sub('', string))

    def format(self, record):
        """
        Formats a record for output.
        Takes a custom formatting path on a terminal.
        :param dict record: input record logging
        :return: str
        """

        if self.is_tty:
            message = self.colorize(record)
        else:
            message = logging.StreamHandler.format(self, record)

        return message



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from src.core import helper
from src.core.http.socks import Socket
from .exceptions import ResponseError
from .providers import ResponseProvider
from src.core.http.plugins.response_plugin import ResponsePlugin, ResponsePluginError


class Response(ResponseProvider):

    """Response class"""

    def __init__(self, config, debug, **kwargs):
        """
        Response instance
        :param src.lib.browser.config.Config config: configurations
        :param src.lib.browser.debug.Debug debug: debugger
        """

        ResponseProvider.__init__(self, config)

        self.__debug = debug
        self.__config = config
        self.__tpl = kwargs.get('tpl')

        if True is self.__config.is_sniff:
            try:
                self.load_sniffers_plugins(self.__config.sniffers)
            except ResponsePluginError as error:
                raise ResponseError(str(error))

    def load_sniffers_plugins(self, plugins):
        """

        :param list plugins: response plugins list
        :return:
        """

        for plugin in plugins:
            try:
                plg_object = ResponsePlugin.load(plugin)
                self._response_plugins.append(plg_object)
                if 0 < self.__debug.level:
                    self.__debug.debug_load_sniffer_plugin(plg_object.DESCRIPTION)
            except ResponsePluginError as error:
                raise ResponsePluginError(str(error))

    def handle(self, response, request_url, items_size, total_size, ignore_list):
        """
        Response handler
        :param urllib3.response.HTTPResponse response: response object
        :param str request_url: url from request
        :param int items_size: current items sizes
        :param int total_size: response object
        :param list ignore_list: ignore list
        :raise ResponseError
        :return: dict
        """

        if hasattr(response, 'status'):
            if self.HTTP_DBG_LEVEL <= self.__debug.level:
                response.headers.update({'Status': str(response.status)})
                if helper.is_jsonable(response.headers.items()):
                    self.__debug.debug_response(response.headers.items())
                else:
                    self.__debug.debug_response(dict(response.headers))

            try:
                status = super(Response, self).detect(request_url, response)
                redirect_uri = None

                if status in ['redirect']:
                    redirect_uri = self._get_redirect_url(request_url, response)
                    check_for_ignored = helper.parse_url(redirect_uri).path[1:]
                    if check_for_ignored in ignore_list:
                        status = 'failed'
                    if self.__config.SUBDOMAINS_SCAN == self.__config.scan:  # subdomains
                        ips = Socket.get_ips_addresses(helper.parse_url(request_url).hostname)
                        url = request_url = '{0} {1}'.format(request_url, ips)
                    else:
                        url = redirect_uri
                else:
                    if self.__config.SUBDOMAINS_SCAN == self.__config.scan:  # subdomains
                        ips = Socket.get_ips_addresses(helper.parse_url(request_url).hostname)
                        url = request_url = '{0} {1}'.format(request_url, ips)
                    else:
                        url = request_url
                self.__debug.debug_request_uri(status=status,
                                               request_uri=request_url,
                                               redirect_uri=redirect_uri,
                                               items_size=items_size,
                                               total_size=total_size,
                                               content_size=ResponseProvider._get_content_size(response)
                                               )

                return status, url

            except Exception as error:
                raise ResponseError(str(error))
        
        elif 'subdomains' in self._cfg.scan:
            status = 'failed'
            self.__debug.debug_request_uri(status=status,
                                           request_uri=request_url,
                                           items_size=items_size,
                                           total_size=total_size,
                                           content_size=ResponseProvider._get_content_size(response)
                                           )
            return status, request_url
        else:
            pass



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import unittest
from src.core.logger.logger import Logger
from io import StringIO
from mock import patch
from src.lib.tpl import Tpl, TplError


class TestTpl(unittest.TestCase):
    """TestTpl class"""
    
    def tearDown(self):
        logger = Logger.log()
        for handler in logger.handlers:
            logger.removeHandler(handler)

    def prompt_answer(self):
        ans = Tpl.prompt(msg='fake')
        return ans

    def test_cancel(self):
        """ Tpl.cancel() test """
    
        with self.assertRaises(SystemExit) as context:
            Tpl.cancel(key='abort')
        self.assertTrue(SystemExit == context.expected)

        
    def test_cancel_exception(self):
        """ Tpl.cancel() exception test """

        undefined = 'undefined'
        with self.assertRaises(TplError) as context:
            Tpl.cancel(key=undefined)
        self.assertTrue(undefined in str(context.exception))
        self.assertTrue(TplError == context.expected)
        
    def test_prompt(self):
        """ Tpl.prompt() test """

        with patch('builtins.input', return_value='fake') as _raw_input:
            self.assertEqual(self.prompt_answer(), 'fake')
            _raw_input.assert_called_once_with('fake')

    def test_prompt_exception(self):
        """ Tpl.prompt() exception test """
        
        undefined = 'undefined'
        with self.assertRaises(TplError) as context:
            Tpl.prompt(key=undefined)
        self.assertTrue(undefined in str(context.exception))
        self.assertTrue(TplError == context.expected)

    def test_line(self):
        """ Tpl.line() test """

        expected = Tpl.line('test')
        self.assertTrue('test' in expected)
    
    def test_line_with_key(self):
        """ Tpl.line() test with key """
        
        expected = Tpl.line(key='report', plugin='testplugin', dest='testdst')
        self.assertTrue('testplugin' in expected)
        self.assertTrue('testdst' in expected)

    def test_line_log(self):
        """ Tpl.line_log() test """

        with patch('sys.stdout', new=StringIO()) as fakeOutput:
            Tpl.line_log('test_line_log')
            self.assertTrue('test_line_log' in fakeOutput.getvalue().strip())
    
    def test_line_log_exception(self):
        """ Tpl.line_log() exception test """

        undefined = 'undefined'
        with self.assertRaises(TplError) as context:
            Tpl.line_log(key=undefined)
        self.assertTrue(undefined in str(context.exception))
        self.assertTrue(TplError == context.expected)
        
    def test_message(self):
        """ Tpl.message() test """

        with patch('sys.stdout', new=StringIO()) as fakeOutput:
            Tpl.message('test_message')
            self.assertTrue('test_message' in fakeOutput.getvalue().strip())

    def test_error(self):
        """ Tpl.error() test """

        with patch('sys.stderr', new=StringIO()) as fakeOutput:
            Tpl.error('test_error')
            self.assertTrue('' in fakeOutput.getvalue().strip())

    def test_warning(self):
        """ Tpl.warning() test """

        with patch('sys.stdout', new=StringIO()) as fakeOutput:
            Tpl.warning('test_warning')
            self.assertTrue('' in fakeOutput.getvalue().strip())

    def test_line_exception(self):
        """ Tpl.line() exception test """

        undefined = 'undefined'
        with self.assertRaises(TplError) as context:
            Tpl.line(key=undefined)
        self.assertTrue(undefined in str(context.exception))
        self.assertTrue(TplError == context.expected)

    def test_info(self):
        """ Tpl.info() test """
    
        with patch('sys.stdout', new=StringIO()) as fakeOutput:
            Tpl.info('test_info')
            self.assertTrue('' in fakeOutput.getvalue().strip())
    
    def test_error_exception(self):
        """ Tpl.error() exception test """

        undefined = 'undefined'
        with self.assertRaises(TplError) as context:
            Tpl.error(key=undefined)
        self.assertTrue(undefined in str(context.exception))
        self.assertTrue(TplError == context.expected)
        
    def test_info_exception(self):
        """ Tpl.info() exception test """

        undefined = 'undefined'
        with self.assertRaises(TplError) as context:
            Tpl.info(key=undefined)
        self.assertTrue(undefined in str(context.exception))
        self.assertTrue(TplError == context.expected)
        
    def test_warning_exception(self):
        """ Tpl.warning() exception test """

        undefined = 'undefined'
        with self.assertRaises(TplError) as context:
            Tpl.warning(key=undefined)
        self.assertTrue(undefined in str(context.exception))
        self.assertTrue(TplError == context.expected)
            
if __name__ == "__main__":
    unittest.main()


# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import re

from src.core import helper
from .exceptions import FilterError


class Filter(object):

    """Filter class"""

    URL_REGEX = "^(\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})|(?:[-A-Za-z0-9]+\.)+([-A-Za-z]|\w){2,8}$"

    @staticmethod
    def filter(args):
        """
        Filter options
        :param dict args:
        :return: dict
        """

        filtered = {}

        for key, value in args.items():
            if 'scan' == key:
                filtered['scan'] = Filter.scan(value)
            elif 'host' == key:
                filtered['host'] = Filter.host(value)
                filtered['scheme'] = Filter.scheme(value)
                filtered['ssl'] = Filter.ssl(filtered['scheme'])
            else:
                if 'proxy' == key:
                    filtered[key] = Filter.proxy(value)
                else:
                    filtered[key] = value

        return filtered

    @staticmethod
    def scheme(hostname):
        """
        Get the scheme of the input hostname.

        :param hostname: A string representing the input hostname.
        :type hostname: str
        :return: A string representing the scheme of the input hostname.
        :rtype: str
        """

        scheme = helper.parse_url(hostname).scheme
        if not scheme:
            scheme = 'http'
        return scheme + "://"

    @staticmethod
    def ssl(scheme):
        """
        If `ssl` in action
        :param str scheme: scheme protocol
        :return: bool
        """

        return 'https://' == scheme

    @staticmethod
    def host(hostname):
        """
        Input `host` param filter
        :param str hostname: input hostname
        :raise FilterError
        :return: str
        """

        if not re.search('http', hostname, re.IGNORECASE):
            if re.search('https', hostname, re.IGNORECASE):
                hostname = "https://" + hostname
            else:
                hostname = "http://" + hostname

        hostname = helper.parse_url(hostname).netloc
        regex = re.compile(r"" + Filter.URL_REGEX + "", re.UNICODE)

        if not regex.match(hostname):
            try:
                hostname = helper.decode_hostname(hostname)
            except UnicodeError as error:
                raise FilterError("\"{0}\" is invalid host. {1}".format(hostname, str(error)))
            if not regex.match(hostname):
                raise FilterError("\"{0}\" is invalid host. Use ip, http(s) or just hostname".format(hostname))
        return hostname

    @staticmethod
    def proxy(proxyaddress):
        """
        Input `proxy` param filter
        :param str proxyaddress: input proxy server address
        :raise FilterError
        :return: str
        """

        proxy = helper.parse_url(proxyaddress)

        if proxy.scheme not in ['http', 'https', 'socks4', 'socks5'] or None is proxy.port:
            raise FilterError("\"{0}\" is invalid proxy in --proxy. Use scheme:ip:port format".format(proxyaddress))
        return proxyaddress

    @staticmethod
    def scan(choose):
        """
        Input `scan` type filter
        :param str choose: preferred scan type
        :return: str
        """

        if choose not in ['directories', 'subdomains']:
            choose = 'directories'
        return choose



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import re
from .provider import ResponsePluginProvider


class IndexofResponsePlugin(ResponsePluginProvider):
    """ IndexofResponsePlugin class"""

    DESCRIPTION = 'IndexOf (detect Index Of/ Apache directories)'
    RESPONSE_INDEX = 'indexof'
    DEFAULT_STATUSES = [100, 101, 200, 201, 202, 203, 204, 205, 206, 207, 208]
    INDEX_OF_TITLE = 'Index of /'

    def __init__(self, void):
        """
        ResponsePluginProvider constructor
        """

        ResponsePluginProvider.__init__(self)

    def process(self, response):
        """
        Process data
        :return: str
        """

        if response.status in self.DEFAULT_STATUSES:
            super().process(response)

            if 0 < len(self._body):
                title = re.search('<title>(.+?)</title>', self._body, re.IGNORECASE | re.DOTALL)
                if None is not title and None is not re.search(self.INDEX_OF_TITLE, title.group(1), re.IGNORECASE):
                    return self.RESPONSE_INDEX
            return None



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

import sys


class Color(object):

    """Color class"""

    default = 'white'

    @staticmethod
    def __get(key):
        """
        Get color key
        :param str key: color name
        :return: int
        """

        colorlist = {'black': 0, 'red': 1, 'green': 2, 'yellow': 3, 'blue': 4, 'magenta': 5, 'cyan': 6, 'white': 7}
        color_text = colorlist[key] if key in colorlist else colorlist[Color.default]
        return color_text

    @staticmethod
    def __has_colors(stream):
        """
        Is tty output check
        :param object stream: input stream
        :return: bool
        """

        if not hasattr(stream, "isatty"):
            return False
        # noinspection PyUnresolvedReferences
        if not stream.isatty():
            return False  # auto color only on TTYs
        # noinspection PyBroadException
        try:
            import curses
            curses.setupterm()
            return curses.tigetnum("colors") > 2
        except Exception:
            # guess false in case of error
            return False

    @staticmethod
    def colored(text, color):
        """
        Output colorized text
        :param str text: message
        :param str color: prefered color
        :return: string
        """

        if Color.__has_colors(sys.stdout):

            if isinstance(text, bytes):
                text = str(text, "utf-8")

            text = text.strip('\n')
            seq = "\x1b[%dm" % (30 + Color.__get(color)) + text + "\x1b[0m"
            return seq
        else:
            return text



# -*- coding: utf-8 -*-

"""
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

    Development Team: Brain Storm Team
"""

from __future__ import absolute_import
import unittest
from ddt import ddt, data
from src.core import CoreConfig
from src import Controller, SrcError


@ddt
class TestController(unittest.TestCase):
    """TestController class"""

    def setUp(self):
        self.controller = Controller.__new__(Controller)
        self.config = CoreConfig

    def tearDown(self):
        del self.controller
        del self.config

    def test_init_exception(self):
        """ Controller.init() exception test """

        self.config['info']['required_versions'] = {
            'minor': '4.0',
            'major': '4.5'
        }
        with self.assertRaises(SrcError) as context:
            Controller().scan_action({})
        self.assertTrue(SrcError == context.expected)

    @data( {'version': True}, {'examples': True}, {'update' : True}) #, {'docs' : True}
    def test_run(self, args):
        """ Controller.run() test """
        
        setattr(self.controller, 'ioargs', args)
        self.controller.run()
        self.assertTrue(True)
        
    def test_run_exception(self):
        """ Controller.run() exception test """

        with self.assertRaises(SrcError) as context:
            self.controller.run()
        self.assertTrue(SrcError == context.expected)
    
    def test_scan_action_exception(self):
        """ Controller.scan_action() exception test """
        
        with self.assertRaises(SrcError) as context:
            self.controller.scan_action(params={
                'host': 'http://example.com',
                'port': 80
            })
        self.assertTrue(SrcError == context.expected)

    def test_examples_action(self):
        """ Controller.examples_action() test """

        self.assertIsNone(self.controller.examples_action())

    def test_update_action(self):
        """ Controller.update_action() test """

        self.assertIsNone(self.controller.update_action())

    def test_version_action(self):
        """ Controller.version_action() test """

        self.assertIsNone(self.controller.version_action())

    def test_local_version(self):
        """ Controller.local_version() test """

        self.assertIsNone(self.controller.local_version())


if __name__ == "__main__":
    unittest.main()
